import type { Chain, Network, Platform, PlatformToChains } from "@wormhole-foundation/sdk-base";
import type { WormholeRegistry } from "./registry.js";
import type { RpcConnection } from "./rpc.js";
import type { ChainsConfig } from "./types.js";
import { Contracts } from "./contracts.js";
/** A string type representing the name of a protocol */
export type ProtocolName = keyof WormholeRegistry.ProtocolToPlatformMapping;
type MappedProtocolPlatforms = keyof WormholeRegistry.ProtocolToPlatformMapping[ProtocolName];
export type EmptyPlatformMap<P extends Platform, PN extends ProtocolName, C extends PlatformToChains<P> = PlatformToChains<P>> = Map<P, ProtocolInitializer<P, PN, C>>;
export type ProtocolImplementation<T extends Platform, PN extends ProtocolName> = PN extends ProtocolName ? T extends MappedProtocolPlatforms ? WormholeRegistry.ProtocolToPlatformMapping[PN][T] : any : never;
export interface ProtocolInitializer<P extends Platform, PN extends ProtocolName, C extends PlatformToChains<P> = PlatformToChains<P>> {
    new (network: Network, chain: C, connection: RpcConnection<P>, contracts: Contracts): ProtocolImplementation<P, PN>;
    fromRpc(rpc: RpcConnection<P>, config: ChainsConfig<Network, P>): Promise<ProtocolImplementation<P, PN>>;
}
export type ProtocolInstance<P extends Platform, PN extends ProtocolName, C extends PlatformToChains<P>> = InstanceType<ProtocolInitializer<P, PN, C>>;
export type ProtocolFactoryMap<PN extends ProtocolName = ProtocolName, P extends Platform = Platform> = Map<PN, Map<P, ProtocolInitializer<P, PN>>>;
export declare function registerProtocol<P extends Platform, PN extends ProtocolName, PI extends ProtocolInitializer<P, PN>>(platform: P, protocol: PN, ctr: PI): void;
export declare function protocolIsRegistered<T extends Platform | Chain, PN extends ProtocolName>(chainOrPlatform: T, protocol: PN): boolean;
export declare function getProtocolInitializer<P extends Platform, PN extends ProtocolName>(platform: P, protocol: PN): ProtocolInitializer<P, PN>;
export declare const create: <N extends "Mainnet" | "Testnet" | "Devnet", P extends "Solana" | "Algorand" | "Near" | "Sui" | "Aptos" | "Btc" | "Evm" | "Cosmwasm", PN extends keyof WormholeRegistry.ProtocolToPlatformMapping, T>(platform: P, protocol: PN, rpc: any, config: ChainsConfig<N, P>) => Promise<T>;
export {};
//# sourceMappingURL=protocol.d.ts.map